@@ -17,11 +17,13 @@ limitations under the License.
 import json
 
 from langtrace.trace_attributes import FrameworkSpanAttributes
+from langtrace_python_sdk.utils.llm import get_span_name
 from opentelemetry import baggage, trace
 from opentelemetry.trace.propagation import set_span_in_context
 
 from opentelemetry.trace import SpanKind
 from opentelemetry.trace.status import Status, StatusCode
+from langtrace.trace_attributes import SpanAttributes
 
 from langtrace_python_sdk.constants.instrumentation.common import (
     LANGTRACE_ADDITIONAL_SPAN_ATTRIBUTES_KEY,
@@ -49,13 +51,15 @@ def generic_patch(
             **(extra_attributes if extra_attributes is not None else {}),
         }
 
+        span_attributes["langchain.metadata"] = to_json_string(kwargs)
+
         if trace_input and len(args) > 0:
             span_attributes["langchain.inputs"] = to_json_string(args)
 
         attributes = FrameworkSpanAttributes(**span_attributes)
 
         with tracer.start_as_current_span(
-            method_name,
+            name=get_span_name(method_name),
             kind=SpanKind.CLIENT,
             context=set_span_in_context(trace.get_current_span()),
         ) as span:
@@ -68,6 +72,16 @@ def generic_patch(
                 if trace_output:
                     span.set_attribute("langchain.outputs", to_json_string(result))
 
+                    prompt_tokens = instance.get_num_tokens(args[0])
+                    completion_tokens = instance.get_num_tokens(result)
+                    if hasattr(result, 'usage'):
+                        prompt_tokens = result.usage.prompt_tokens
+                        completion_tokens = result.usage.completion_tokens
+
+                    span.set_attribute(SpanAttributes.LLM_USAGE_COMPLETION_TOKENS, prompt_tokens)
+                    span.set_attribute(SpanAttributes.LLM_USAGE_PROMPT_TOKENS, completion_tokens)
+
+
                 span.set_status(StatusCode.OK)
                 return result
             except Exception as err:
@@ -85,15 +99,31 @@ def generic_patch(
 
 def clean_empty(d):
     """Recursively remove empty lists, empty dicts, or None elements from a dictionary."""
-    if not isinstance(d, (dict, list)):
+    if not isinstance(d, (dict, list, tuple)):
         return d
+    if isinstance(d, tuple):
+        return tuple(val for val in (clean_empty(val) for val in d) if val != () and val is not None)
     if isinstance(d, list):
-        return [v for v in (clean_empty(v) for v in d) if v != [] and v is not None]
-    return {
-        k: v
-        for k, v in ((k, clean_empty(v)) for k, v in d.items())
-        if v is not None and v != {}
-    }
+        return [val for val in (clean_empty(val) for val in d) if val != [] and val is not None]
+    result = {}
+    for k, val in d.items():
+        if isinstance(val, dict):
+            val = clean_empty(val)
+            if val != {} and val is not None:
+                result[k] = val
+        elif isinstance(val, list):
+            val = [clean_empty(value) for value in val]
+            if val != [] and val is not None:
+                result[k] = val
+        elif isinstance(val, str) and val is not None:
+            if val.strip() != "":
+                result[k] = val.strip()
+        elif isinstance(val, object):
+            # some langchain objects have a text attribute
+            val = getattr(val, 'text', None)
+            if val is not None and val.strip() != "":
+                result[k] = val.strip()
+    return result
 
 
 def custom_serializer(obj):
@@ -108,5 +138,12 @@ def custom_serializer(obj):
 
 def to_json_string(any_object):
     """Converts any object to a JSON-parseable string, omitting empty or None values."""
-    cleaned_object = clean_empty(any_object)
-    return json.dumps(cleaned_object, default=custom_serializer, indent=2)
+    try:
+        cleaned_object = clean_empty(any_object)
+        return json.dumps(cleaned_object, default=custom_serializer, indent=2)
+    except NotImplementedError:
+        # Handle specific types that raise this error
+        return str(any_object)  # or another appropriate fallback
+    except TypeError:
+        # Handle cases where obj is not serializable
+        return str(any_object)
